/////////////////////////////////////////////////////////////////////////////////
// Welcome to Rice University CMC Lab's Wireless open-Access Research Platform //
// (WARP) peripheral test program. The purpose of this program is two-fold,    //
// the first of which, is to make sure the hardware on the board is functional.//
// Secondarily, this is meant to serve as an introduction to learning how to   //
// to program the board.                                                       //
//                                                                             //
// System designed for board type: WARP FPGA Board Ver 1.1                     //
//                                                                             //
// Program written by Scott Novich (snovich@rice.edu)  1.[401].497.2333		    //												   
//	Converted for use in XPS 8.1 by Austin Bratton 										 //
//                                  (bratton@rice.edu) 1.[903].278.7621 	    // 
//																			   						 //
// Version 1.0A 															    					 //
// Last Update Made on 01/18/2006                                              //
//																			   						 //
/////////////////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "xparameters.h"
#include "warplib.h"
#include "xgpio_l.h"
#include "xgpio.h"
#include "xexception_l.h"
#include "xintc_l.h"
#include "xuartlite_l.h"  

#include <unistd.h>


// SUGGESTED READING/REFERENCES:
//
// http://direct.xilinx.com/bvdocs/appnotes/xapp778.pdf -> creating interrupt based systems
//
// *:\EDK\doc\xilinx_drivers_api_toc.htm -> EDK Driver API with in-depth EDK function definitions


XGpio Gpio; // GPIO instance -- must be declared for gpio-based interrupts


// This function is the push button interrupt handler. When a push-button interrupt
// is registered in the system, this function defines what actions should occur.
void pb_int_handler(void *baseaddr_p)
{
		
	Xuint32 pbchoice;

	pbchoice = XGpio_mGetDataReg(XPAR_PUSH_BUTTONS_4BIT_BASEADDR, 2);

	switch(pbchoice)
	{
		case(0x1) : print("\r\nPush Button 2: My, aren't we pushy today?\r\n");
					   break;
		case(0x2) : print("\r\nPush Button 3: Pardon the interruption...\r\n");
					   break;
		case(0x4) : print("\r\nPush Button 4: Push push push...Push push push...\r\n");
					   break;
		case(0x8) : print("\r\nPush Button 5: Hey!  That tickles!\r\n");
					   break;

		default : ; break; // We get rid of the default case because
		                   // the push button actually registers
						       // two interrupts at once (one on push down, and one on pop up)
	}


	// clear the interrupt vector. Once cleared it can be interrupted again.
	XGpio_InterruptClear(&Gpio, 0x7); // clear the interrupt

}


// this function call registers the push button interrupts to test them.
void pbtest()
{

	print("\r\n\r\nEntering Push Button Test Test.\r\n");
	print("Hit PB2, PB3, PB4, or PB5 to output a message.\r\n Hit PB1 a.k.a. 'reset' to exit and return to main.\r\n");

  // The following are function calls required for setting up Push Button interrupts.
  // It is HIGHLY suggested you select these as desired interrupts in the Base System Builder,
  // otherwise there are some complicated steps to jump through. If you are adding the interrupt
  // controller in manually, look at the MHS file for this project. 
  // IT IS ALSO REQUIRED, that you double-click the PushButton instance in the System Assembly view 
  // and make sure that "Support Interrupt" is checked


   // Initialize the push buttons
	XGpio_Initialize(&Gpio, XPAR_PUSH_BUTTONS_4BIT_DEVICE_ID);

	// Initialize exception handling.
	XExc_Init();

	// Register the Intc handler with the exception handler.
	XExc_RegisterHandler(XEXC_ID_NON_CRITICAL_INT, (XExceptionHandler) XIntc_DeviceInterruptHandler, (void *)XPAR_OPB_INTC_0_DEVICE_ID);

	// Register the individual handlers with Intc
	XIntc_RegisterHandler(XPAR_OPB_INTC_0_BASEADDR, XPAR_OPB_INTC_0_PUSH_BUTTONS_4BIT_IP2INTC_IRPT_INTR, (XInterruptHandler)pb_int_handler, (void *) 0);

	// Enable the intc to accept interrupts
	XIntc_mMasterEnable(XPAR_OPB_INTC_0_BASEADDR);

	// Enable each of the individual interrupts
	XIntc_mEnableIntr(XPAR_OPB_INTC_0_BASEADDR,XPAR_PUSH_BUTTONS_4BIT_IP2INTC_IRPT_MASK);

	// set the date direction as input
	XGpio_mSetDataDirection(XPAR_PUSH_BUTTONS_4BIT_BASEADDR,1,0x7);

	// Enable interrupts for the push buttons
	XGpio_InterruptEnable(&Gpio, XGPIO_IR_CH1_MASK);

	// Enable global interrupts. Both this and above line necessary to enable interrupts
	XGpio_InterruptGlobalEnable(&Gpio);


	// Enable exceptionsf
	XExc_mEnableExceptions(XEXC_NON_CRITICAL);

	// wait for interrupt
	while(1)
	{
	}
}


// this function tests the LEDs on the board by toggling/holding them on and off.
void ledtest()
{
	XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, 0x0);

	
	Xuint8 lednum; // for the case statement
	Xuint32 LEDstate = 0x0; // default LED state (all LEDs OFF)
	int j, k=4000000; // k can be adjusted for more or less delay between LED switching in startup.

	
	print("\r\n\r\nEntering LED Test.\r\n");
	print("Enter 1-4 to toggle LEDs(1-4) on and off.\r\n Enter 'q' to exit and return to main.\r\n");


	lednum = XUartLite_RecvByte(STDIN_BASEADDRESS);

	XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, 0x1); //Initialize the LEDs

	for (j=0; j < k; j++);						       //Wait to see that 	  

	XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, 0x2);

	for (j=0; j < k; j++);

	XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, 0x4);

	for (j=0; j < k; j++);

	XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, 0x8);

	for (j=0; j < k; j++);



	while(lednum != 'q' && lednum != 'Q')
	{
			switch(lednum)
			{
	
				// each LED has a spot in the data register for use with the XGpio_mSetDataReg call
				// for instance, LED 1 is tied to spot 0b000*, LED is tied to 0b00*0 etc. IN OTHER WORDS
				// each LED is tied to its own binary digit.
				// to turn it on, it only needs to be set to 1 -- and 0 to turn it off.
				// the following case statement will mask the current reg holdings and add/subtract
				// the proper hex val to cause an LED toggle
				
				case('1') :
					if ((LEDstate&0x1)==0)
					{
						LEDstate = LEDstate + 0x1;
						XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, LEDstate);
					}
					else
					{
						LEDstate = LEDstate - 0x1;
						XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, LEDstate);
					}
	
					break;
				
				case('2') :
					if ((LEDstate&0x2)==0)
					{
						LEDstate = LEDstate + 0x2;
						XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, LEDstate);
					}
					else
					{
						LEDstate = LEDstate - 0x2;
						XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, LEDstate);
					}
					break;
	
				
				case('3') :
					if ((LEDstate&0x4)==0)
					{
						LEDstate = LEDstate + 0x4;
						XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, LEDstate);
					}
					else
					{
						LEDstate = LEDstate - 0x4;
						XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, LEDstate);
					}
					break;
	
	
			
				case('4') :
					if ((LEDstate&0x8)==0)
					{
						LEDstate = LEDstate + 0x8;
						XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, LEDstate);
					}
					else
					{
						LEDstate = LEDstate - 0x8;
						XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, LEDstate);
					}
					break;
	
				
				default : print("\r\nInvalid Choice\r\n"); break;
	
			}
		
		lednum = XUartLite_RecvByte(STDIN_BASEADDRESS);
	}

	print("\r\n\r\nQUIT LED Test program.\r\n\r\n");
}

// this function tests the 7 segment display by performing a basic scroll
// it calls the WARP-specific hex-to-seven-segment conversion function
// from warplib.h
void seg7test()
{
	print("\r\n\r\nEntering 7 Segment Display Test.\r\n\r\n");

	Xuint32 seg1 = XPAR_LED_7SEGMENT_BASEADDR; // shorten the addr names found in xparameters.h
	Xuint32 seg2 = XPAR_LED_7SEGMENT_1_BASEADDR;
	Xuint8 input;
	Xuint8 prev;

	// This is same function call used for the LED test, but a 7-segment decoding is needed.
	// We have created a function call in warplib.h that will do this translation for you.
	// You may also wish to edit it to add other characters you may desire.

	print("Enter a valid HEX value 0->9 or A->F to watch it scroll across the 7 Segment Displays.\r\n Enter 'q' to exit and return to main.\r\n");

	XGpio_mSetDataReg(seg1, 1, WarpLib_segmap(2));
	XGpio_mSetDataReg(seg2, 1, WarpLib_segmap(2));
	
	input = XUartLite_RecvByte(STDIN_BASEADDRESS);

	while (input!='q' && input!='Q')
	{
	   XGpio_mSetDataReg(seg1, 1, WarpLib_segmap(input));
		XGpio_mSetDataReg(seg2, 1, WarpLib_segmap(prev));
		prev = input;
				
		input = XUartLite_RecvByte(STDIN_BASEADDRESS);
	}
	print("\r\n\r\nQUIT 7 Segment Display Test program.\r\n\r\n");
}

// The following test the dip switch buttons located near the LEDs, push-buttons, and 7-segment display
// by assigning each dip switch to to toggle an LED.  
void dipswtest()
{
	print("\n\r\n\rEntering Dip Switch Test...\r\n\r\n");
	

	Xuint32 XDIPSTATE = 0x0;

	char DIPSTATE;

	

	Xuint32 seg1 = XPAR_LED_7SEGMENT_BASEADDR; // shorten the addr names found in xparameters.h
	Xuint32 seg2 = XPAR_LED_7SEGMENT_1_BASEADDR;

	Xuint8 testchoice = 0;	

	

	print("\r\nSet the Dip Switches as you please and press any key.\r\n  Press 'q' to return to the Main Menu\r\n\r\n");

	testchoice = XUartLite_RecvByte(STDIN_BASEADDRESS);

	

	while(testchoice != 'q' && testchoice != 'Q')

	{

	

		XDIPSTATE = XGpio_mGetDataReg(XPAR_DIPSWS_4BIT_BASEADDR, 2); // try 1 if this doesn't work.

		DIPSTATE = (char)XDIPSTATE;

		

	   XGpio_mSetDataReg(seg1, 1, WarpLib_segmap(DIPSTATE));  // Change 7-Segment Display
		XGpio_mSetDataReg(seg2, 1, WarpLib_segmap(DIPSTATE));

		

		XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, XDIPSTATE); // Output to LEDs 

		

			testchoice = XUartLite_RecvByte(STDIN_BASEADDRESS);

	}

		

}

int main()
{

	XGpio_mSetDataReg(XPAR_LEDS_4BIT_BASEADDR, 1, 0x1); // turn an LED on for debugging purposes
														// program DLs succesfully if LED 1 on the board
														// is lit.

	Xuint8 testchoice = 32; // Initialize testchoice
	

	print("\n\r\n\rRice University WARP Board Peripheral Test Program\r\n\r\n");

	while(1)
	{

			print("\r\nMENU\r\n\r\n");
			print("(1): Pushbuttons Test\r\n");
			print("(2): LEDs Test\r\n");
			print("(3): 7 Segment Displays Test\r\n");
			print("(4): Dip Switch Test\r\n");
			print("Please Enter Number corresponding to test choice:\r\n");

		
			testchoice = XUartLite_RecvByte(STDIN_BASEADDRESS); 

			switch(testchoice)
			{
				case('1') : pbtest(); break;
				case('2') : ledtest(); break;
				case('3') : seg7test(); break;
				case('4') : dipswtest(); break;
				default : print("\r\n\r\nInvalid Choice Entered\r\n\r\n"); break;
			}
	}
		return 0;
}
